@@ -1,52 +1,32 @@
-import pytest
-import redis
 import bz2
 import csv
-import time
 import os
-
+import time
 from io import TextIOWrapper
-from .conftest import skip_ifmodversion_lt, default_redismod_url
-from redis import Redis
 
+import pytest
+
+import redis
 import redis.commands.search
-from redis.commands.json.path import Path
-from redis.commands.search import Search
-from redis.commands.search.field import (
-    GeoField,
-    NumericField,
-    TagField,
-    TextField
-)
-from redis.commands.search.query import (
-    GeoFilter,
-    NumericFilter,
-    Query
-)
-from redis.commands.search.result import Result
-from redis.commands.search.indexDefinition import IndexDefinition, IndexType
-from redis.commands.search.suggestion import Suggestion
 import redis.commands.search.aggregation as aggregations
 import redis.commands.search.reducers as reducers
+from redis import Redis
+from redis.commands.json.path import Path
+from redis.commands.search import Search
+from redis.commands.search.field import GeoField, NumericField, TagField, TextField
+from redis.commands.search.indexDefinition import IndexDefinition, IndexType
+from redis.commands.search.query import GeoFilter, NumericFilter, Query
+from redis.commands.search.result import Result
+from redis.commands.search.suggestion import Suggestion
 
-WILL_PLAY_TEXT = (
-    os.path.abspath(
-        os.path.join(
-            os.path.dirname(__file__),
-            "testdata",
-            "will_play_text.csv.bz2"
-        )
-    )
+from .conftest import default_redismod_url, skip_ifmodversion_lt
+
+WILL_PLAY_TEXT = os.path.abspath(
+    os.path.join(os.path.dirname(__file__), "testdata", "will_play_text.csv.bz2")
 )
 
-TITLES_CSV = (
-    os.path.abspath(
-        os.path.join(
-            os.path.dirname(__file__),
-            "testdata",
-            "titles.csv"
-        )
-    )
+TITLES_CSV = os.path.abspath(
+    os.path.join(os.path.dirname(__file__), "testdata", "titles.csv")
 )
 
 
@@ -81,9 +61,7 @@ def getClient():
 def createIndex(client, num_docs=100, definition=None):
     try:
         client.create_index(
-            (TextField("play", weight=5.0),
-                TextField("txt"),
-                NumericField("chapter")),
+            (TextField("play", weight=5.0), TextField("txt"), NumericField("chapter")),
             definition=definition,
         )
     except redis.ResponseError:
@@ -96,10 +74,9 @@ def createIndex(client, num_docs=100, definition=None):
     r = csv.reader(bzfp, delimiter=";")
     for n, line in enumerate(r):
 
-        play, chapter, _, text = \
-            line[1], line[2], line[4], line[5]
+        play, chapter, _, text = line[1], line[2], line[4], line[5]
 
-        key = "{}:{}".format(play, chapter).lower()
+        key = f"{play}:{chapter}".lower()
         d = chapters.setdefault(key, {})
         d["play"] = play
         d["txt"] = d.get("txt", "") + " " + text
@@ -183,12 +160,10 @@ def test_client(client):
 
     # test in fields
     txt_total = (
-        client.ft().search(
-            Query("henry").no_content().limit_fields("txt")).total
+        client.ft().search(Query("henry").no_content().limit_fields("txt")).total
     )
     play_total = (
-        client.ft().search(
-            Query("henry").no_content().limit_fields("play")).total
+        client.ft().search(Query("henry").no_content().limit_fields("play")).total
     )
     both_total = (
         client.ft()
@@ -217,10 +192,8 @@ def test_client(client):
 
     # test slop and in order
     assert 193 == client.ft().search(Query("henry king")).total
-    assert 3 == client.ft().search(
-        Query("henry king").slop(0).in_order()).total
-    assert 52 == client.ft().search(
-        Query("king henry").slop(0).in_order()).total
+    assert 3 == client.ft().search(Query("henry king").slop(0).in_order()).total
+    assert 52 == client.ft().search(Query("king henry").slop(0).in_order()).total
     assert 53 == client.ft().search(Query("henry king").slop(0)).total
     assert 167 == client.ft().search(Query("henry king").slop(100)).total
 
@@ -284,11 +257,7 @@ def test_replace(client):
 
     res = client.ft().search("foo bar")
     assert 2 == res.total
-    client.ft().add_document(
-        "doc1",
-        replace=True,
-        txt="this is a replaced doc"
-    )
+    client.ft().add_document("doc1", replace=True, txt="this is a replaced doc")
 
     res = client.ft().search("foo bar")
     assert 1 == res.total
@@ -301,10 +270,7 @@ def test_replace(client):
 
 @pytest.mark.redismod
 def test_stopwords(client):
-    client.ft().create_index(
-        (TextField("txt"),),
-        stopwords=["foo", "bar", "baz"]
-    )
+    client.ft().create_index((TextField("txt"),), stopwords=["foo", "bar", "baz"])
     client.ft().add_document("doc1", txt="foo bar")
     client.ft().add_document("doc2", txt="hello world")
     waitForIndex(client, "idx")
@@ -318,17 +284,8 @@ def test_stopwords(client):
 
 @pytest.mark.redismod
 def test_filters(client):
-    client.ft().create_index(
-        (TextField("txt"),
-            NumericField("num"),
-            GeoField("loc"))
-    )
-    client.ft().add_document(
-        "doc1",
-        txt="foo bar",
-        num=3.141,
-        loc="-0.441,51.458"
-    )
+    client.ft().create_index((TextField("txt"), NumericField("num"), GeoField("loc")))
+    client.ft().add_document("doc1", txt="foo bar", num=3.141, loc="-0.441,51.458")
     client.ft().add_document("doc2", txt="foo baz", num=2, loc="-0.1,51.2")
 
     waitForIndex(client, "idx")
@@ -336,8 +293,7 @@ def test_filters(client):
     q1 = Query("foo").add_filter(NumericFilter("num", 0, 2)).no_content()
     q2 = (
         Query("foo")
-        .add_filter(
-            NumericFilter("num", 2, NumericFilter.INF, minExclusive=True))
+        .add_filter(NumericFilter("num", 2, NumericFilter.INF, minExclusive=True))
         .no_content()
     )
     res1, res2 = client.ft().search(q1), client.ft().search(q2)
@@ -348,10 +304,8 @@ def test_filters(client):
     assert "doc1" == res2.docs[0].id
 
     # Test geo filter
-    q1 = Query("foo").add_filter(
-        GeoFilter("loc", -0.44, 51.45, 10)).no_content()
-    q2 = Query("foo").add_filter(
-        GeoFilter("loc", -0.44, 51.45, 100)).no_content()
+    q1 = Query("foo").add_filter(GeoFilter("loc", -0.44, 51.45, 10)).no_content()
+    q2 = Query("foo").add_filter(GeoFilter("loc", -0.44, 51.45, 100)).no_content()
     res1, res2 = client.ft().search(q1), client.ft().search(q2)
 
     assert 1 == res1.total
@@ -377,10 +331,7 @@ def test_payloads_with_no_content(client):
 
 @pytest.mark.redismod
 def test_sort_by(client):
-    client.ft().create_index(
-        (TextField("txt"),
-            NumericField("num", sortable=True))
-    )
+    client.ft().create_index((TextField("txt"), NumericField("num", sortable=True)))
     client.ft().add_document("doc1", txt="foo bar", num=1)
     client.ft().add_document("doc2", txt="foo baz", num=2)
     client.ft().add_document("doc3", txt="foo qux", num=3)
@@ -422,10 +373,7 @@ def test_drop_index():
 @pytest.mark.redismod
 def test_example(client):
     # Creating the index definition and schema
-    client.ft().create_index(
-        (TextField("title", weight=5.0),
-            TextField("body"))
-    )
+    client.ft().create_index((TextField("title", weight=5.0), TextField("body")))
 
     # Indexing a document
     client.ft().add_document(
@@ -483,12 +431,7 @@ def test_auto_complete(client):
     client.ft().sugadd("ac", Suggestion("pay2", payload="pl2"))
     client.ft().sugadd("ac", Suggestion("pay3", payload="pl3"))
 
-    sugs = client.ft().sugget(
-        "ac",
-        "pay",
-        with_payloads=True,
-        with_scores=True
-    )
+    sugs = client.ft().sugget("ac", "pay", with_payloads=True, with_scores=True)
     assert 3 == len(sugs)
     for sug in sugs:
         assert sug.payload
@@ -550,11 +493,7 @@ def test_no_index(client):
 
 @pytest.mark.redismod
 def test_partial(client):
-    client.ft().create_index(
-        (TextField("f1"),
-            TextField("f2"),
-            TextField("f3"))
-    )
+    client.ft().create_index((TextField("f1"), TextField("f2"), TextField("f3")))
     client.ft().add_document("doc1", f1="f1_val", f2="f2_val")
     client.ft().add_document("doc2", f1="f1_val", f2="f2_val")
     client.ft().add_document("doc1", f3="f3_val", partial=True)
@@ -572,11 +511,7 @@ def test_partial(client):
 
 @pytest.mark.redismod
 def test_no_create(client):
-    client.ft().create_index(
-        (TextField("f1"),
-            TextField("f2"),
-            TextField("f3"))
-    )
+    client.ft().create_index((TextField("f1"), TextField("f2"), TextField("f3")))
     client.ft().add_document("doc1", f1="f1_val", f2="f2_val")
     client.ft().add_document("doc2", f1="f1_val", f2="f2_val")
     client.ft().add_document("doc1", f3="f3_val", no_create=True)
@@ -592,21 +527,12 @@ def test_no_create(client):
     assert 1 == res.total
 
     with pytest.raises(redis.ResponseError):
-        client.ft().add_document(
-            "doc3",
-            f2="f2_val",
-            f3="f3_val",
-            no_create=True
-        )
+        client.ft().add_document("doc3", f2="f2_val", f3="f3_val", no_create=True)
 
 
 @pytest.mark.redismod
 def test_explain(client):
-    client.ft().create_index(
-        (TextField("f1"),
-            TextField("f2"),
-            TextField("f3"))
-    )
+    client.ft().create_index((TextField("f1"), TextField("f2"), TextField("f3")))
     res = client.ft().explain("@f3:f3_val @f2:f2_val @f1:f1_val")
     assert res
 
@@ -786,11 +712,7 @@ def test_alter_schema_add(client):
 def test_spell_check(client):
     client.ft().create_index((TextField("f1"), TextField("f2")))
 
-    client.ft().add_document(
-        "doc1",
-        f1="some valid content",
-        f2="this is sample text"
-    )
+    client.ft().add_document("doc1", f1="some valid content", f2="this is sample text")
     client.ft().add_document("doc2", f1="very important", f2="lorem ipsum")
     waitForIndex(client, "idx")
 
@@ -861,7 +783,7 @@ def test_phonetic_matcher(client):
 
     res = client.ft().search(Query("Jon"))
     assert 2 == len(res.docs)
-    assert ["John", "Jon"] == sorted([d.name for d in res.docs])
+    assert ["John", "Jon"] == sorted(d.name for d in res.docs)
 
 
 @pytest.mark.redismod
@@ -881,8 +803,7 @@ def test_scorer(client):
     assert 1.0 == res.docs[0].score
     res = client.ft().search(Query("quick").scorer("TFIDF").with_scores())
     assert 1.0 == res.docs[0].score
-    res = client.ft().search(
-        Query("quick").scorer("TFIDF.DOCNORM").with_scores())
+    res = client.ft().search(Query("quick").scorer("TFIDF.DOCNORM").with_scores())
     assert 0.1111111111111111 == res.docs[0].score
     res = client.ft().search(Query("quick").scorer("BM25").with_scores())
     assert 0.17699114465425977 == res.docs[0].score
@@ -930,7 +851,7 @@ def test_config(client):
 
 
 @pytest.mark.redismod
-def test_aggregations(client):
+def test_aggregations_groupby(client):
     # Creating the index definition and schema
     client.ft().create_index(
         (
@@ -967,36 +888,226 @@ def test_aggregations(client):
     req = aggregations.AggregateRequest("redis").group_by(
         "@parent",
         reducers.count(),
-        reducers.count_distinct("@title"),
-        reducers.count_distinctish("@title"),
-        reducers.sum("@random_num"),
-        reducers.min("@random_num"),
-        reducers.max("@random_num"),
-        reducers.avg("@random_num"),
-        reducers.stddev("random_num"),
-        reducers.quantile("@random_num", 0.5),
-        reducers.tolist("@title"),
-        reducers.first_value("@title"),
-        reducers.random_sample("@title", 2),
     )
 
-    res = client.ft().aggregate(req)
+    res = client.ft().aggregate(req).rows[0]
+    assert res[1] == "redis"
+    assert res[3] == "3"
 
-    res = res.rows[0]
-    assert len(res) == 26
-    assert "redis" == res[1]
-    assert "3" == res[3]
-    assert "3" == res[5]
-    assert "3" == res[7]
-    assert "21" == res[9]
-    assert "3" == res[11]
-    assert "10" == res[13]
-    assert "7" == res[15]
-    assert "3.60555127546" == res[17]
-    assert "10" == res[19]
-    assert ["RediSearch", "RedisAI", "RedisJson"] == res[21]
-    assert "RediSearch" == res[23]
-    assert 2 == len(res[25])
+    req = aggregations.AggregateRequest("redis").group_by(
+        "@parent",
+        reducers.count_distinct("@title"),
+    )
+
+    res = client.ft().aggregate(req).rows[0]
+    assert res[1] == "redis"
+    assert res[3] == "3"
+
+    req = aggregations.AggregateRequest("redis").group_by(
+        "@parent",
+        reducers.count_distinctish("@title"),
+    )
+
+    res = client.ft().aggregate(req).rows[0]
+    assert res[1] == "redis"
+    assert res[3] == "3"
+
+    req = aggregations.AggregateRequest("redis").group_by(
+        "@parent",
+        reducers.sum("@random_num"),
+    )
+
+    res = client.ft().aggregate(req).rows[0]
+    assert res[1] == "redis"
+    assert res[3] == "21"  # 10+8+3
+
+    req = aggregations.AggregateRequest("redis").group_by(
+        "@parent",
+        reducers.min("@random_num"),
+    )
+
+    res = client.ft().aggregate(req).rows[0]
+    assert res[1] == "redis"
+    assert res[3] == "3"  # min(10,8,3)
+
+    req = aggregations.AggregateRequest("redis").group_by(
+        "@parent",
+        reducers.max("@random_num"),
+    )
+
+    res = client.ft().aggregate(req).rows[0]
+    assert res[1] == "redis"
+    assert res[3] == "10"  # max(10,8,3)
+
+    req = aggregations.AggregateRequest("redis").group_by(
+        "@parent",
+        reducers.avg("@random_num"),
+    )
+
+    res = client.ft().aggregate(req).rows[0]
+    assert res[1] == "redis"
+    assert res[3] == "7"  # (10+3+8)/3
+
+    req = aggregations.AggregateRequest("redis").group_by(
+        "@parent",
+        reducers.stddev("random_num"),
+    )
+
+    res = client.ft().aggregate(req).rows[0]
+    assert res[1] == "redis"
+    assert res[3] == "3.60555127546"
+
+    req = aggregations.AggregateRequest("redis").group_by(
+        "@parent",
+        reducers.quantile("@random_num", 0.5),
+    )
+
+    res = client.ft().aggregate(req).rows[0]
+    assert res[1] == "redis"
+    assert res[3] == "10"
+
+    req = aggregations.AggregateRequest("redis").group_by(
+        "@parent",
+        reducers.tolist("@title"),
+    )
+
+    res = client.ft().aggregate(req).rows[0]
+    assert res[1] == "redis"
+    assert res[3] == ["RediSearch", "RedisAI", "RedisJson"]
+
+    req = aggregations.AggregateRequest("redis").group_by(
+        "@parent",
+        reducers.first_value("@title").alias("first"),
+    )
+
+    res = client.ft().aggregate(req).rows[0]
+    assert res == ["parent", "redis", "first", "RediSearch"]
+
+    req = aggregations.AggregateRequest("redis").group_by(
+        "@parent",
+        reducers.random_sample("@title", 2).alias("random"),
+    )
+
+    res = client.ft().aggregate(req).rows[0]
+    assert res[1] == "redis"
+    assert res[2] == "random"
+    assert len(res[3]) == 2
+    assert res[3][0] in ["RediSearch", "RedisAI", "RedisJson"]
+
+
+@pytest.mark.redismod
+def test_aggregations_sort_by_and_limit(client):
+    client.ft().create_index(
+        (
+            TextField("t1"),
+            TextField("t2"),
+        )
+    )
+
+    client.ft().client.hset("doc1", mapping={"t1": "a", "t2": "b"})
+    client.ft().client.hset("doc2", mapping={"t1": "b", "t2": "a"})
+
+    # test sort_by using SortDirection
+    req = aggregations.AggregateRequest("*").sort_by(
+        aggregations.Asc("@t2"), aggregations.Desc("@t1")
+    )
+    res = client.ft().aggregate(req)
+    assert res.rows[0] == ["t2", "a", "t1", "b"]
+    assert res.rows[1] == ["t2", "b", "t1", "a"]
+
+    # test sort_by without SortDirection
+    req = aggregations.AggregateRequest("*").sort_by("@t1")
+    res = client.ft().aggregate(req)
+    assert res.rows[0] == ["t1", "a"]
+    assert res.rows[1] == ["t1", "b"]
+
+    # test sort_by with max
+    req = aggregations.AggregateRequest("*").sort_by("@t1", max=1)
+    res = client.ft().aggregate(req)
+    assert len(res.rows) == 1
+
+    # test limit
+    req = aggregations.AggregateRequest("*").sort_by("@t1").limit(1, 1)
+    res = client.ft().aggregate(req)
+    assert len(res.rows) == 1
+    assert res.rows[0] == ["t1", "b"]
+
+
+@pytest.mark.redismod
+def test_aggregations_load(client):
+    client.ft().create_index(
+        (
+            TextField("t1"),
+            TextField("t2"),
+        )
+    )
+
+    client.ft().client.hset("doc1", mapping={"t1": "hello", "t2": "world"})
+
+    # load t1
+    req = aggregations.AggregateRequest("*").load("t1")
+    res = client.ft().aggregate(req)
+    assert res.rows[0] == ["t1", "hello"]
+
+    # load t2
+    req = aggregations.AggregateRequest("*").load("t2")
+    res = client.ft().aggregate(req)
+    assert res.rows[0] == ["t2", "world"]
+
+    # load all
+    req = aggregations.AggregateRequest("*").load()
+    res = client.ft().aggregate(req)
+    assert res.rows[0] == ["t1", "hello", "t2", "world"]
+
+
+@pytest.mark.redismod
+def test_aggregations_apply(client):
+    client.ft().create_index(
+        (
+            TextField("PrimaryKey", sortable=True),
+            NumericField("CreatedDateTimeUTC", sortable=True),
+        )
+    )
+
+    client.ft().client.hset(
+        "doc1",
+        mapping={"PrimaryKey": "9::362330", "CreatedDateTimeUTC": "637387878524969984"},
+    )
+    client.ft().client.hset(
+        "doc2",
+        mapping={"PrimaryKey": "9::362329", "CreatedDateTimeUTC": "637387875859270016"},
+    )
+
+    req = aggregations.AggregateRequest("*").apply(
+        CreatedDateTimeUTC="@CreatedDateTimeUTC * 10"
+    )
+    res = client.ft().aggregate(req)
+    assert res.rows[0] == ["CreatedDateTimeUTC", "6373878785249699840"]
+    assert res.rows[1] == ["CreatedDateTimeUTC", "6373878758592700416"]
+
+
+@pytest.mark.redismod
+def test_aggregations_filter(client):
+    client.ft().create_index(
+        (
+            TextField("name", sortable=True),
+            NumericField("age", sortable=True),
+        )
+    )
+
+    client.ft().client.hset("doc1", mapping={"name": "bar", "age": "25"})
+    client.ft().client.hset("doc2", mapping={"name": "foo", "age": "19"})
+
+    req = aggregations.AggregateRequest("*").filter("@name=='foo' && @age < 20")
+    res = client.ft().aggregate(req)
+    assert len(res.rows) == 1
+    assert res.rows[0] == ["name", "foo", "age", "19"]
+
+    req = aggregations.AggregateRequest("*").filter("@age > 15").sort_by("@age")
+    res = client.ft().aggregate(req)
+    assert len(res.rows) == 2
+    assert res.rows[0] == ["age", "19"]
+    assert res.rows[1] == ["age", "25"]
 
 
 @pytest.mark.redismod
@@ -1043,6 +1154,72 @@ def test_index_definition(client):
     createIndex(client.ft(), num_docs=500, definition=definition)
 
 
+@pytest.mark.redismod
+def testExpire(client):
+    client.ft().create_index((TextField("txt", sortable=True),), temporary=4)
+    ttl = client.execute_command("ft.debug", "TTL", "idx")
+    assert ttl > 2
+
+    while ttl > 2:
+        ttl = client.execute_command("ft.debug", "TTL", "idx")
+        time.sleep(0.01)
+
+    # add document - should reset the ttl
+    client.ft().add_document("doc", txt="foo bar", text="this is a simple test")
+    ttl = client.execute_command("ft.debug", "TTL", "idx")
+    assert ttl > 2
+    try:
+        while True:
+            ttl = client.execute_command("ft.debug", "TTL", "idx")
+            time.sleep(0.5)
+    except redis.exceptions.ResponseError:
+        assert ttl == 0
+
+
+@pytest.mark.redismod
+def testSkipInitialScan(client):
+    client.hset("doc1", "foo", "bar")
+    q = Query("@foo:bar")
+
+    client.ft().create_index((TextField("foo"),), skip_initial_scan=True)
+    assert 0 == client.ft().search(q).total
+
+
+@pytest.mark.redismod
+def testSummarizeDisabled_nooffset(client):
+    client.ft().create_index((TextField("txt"),), no_term_offsets=True)
+    client.ft().add_document("doc1", txt="foo bar")
+    with pytest.raises(Exception):
+        client.ft().search(Query("foo").summarize(fields=["txt"]))
+
+
+@pytest.mark.redismod
+def testSummarizeDisabled_nohl(client):
+    client.ft().create_index((TextField("txt"),), no_highlight=True)
+    client.ft().add_document("doc1", txt="foo bar")
+    with pytest.raises(Exception):
+        client.ft().search(Query("foo").summarize(fields=["txt"]))
+
+
+@pytest.mark.redismod
+def testMaxTextFields(client):
+    # Creating the index definition
+    client.ft().create_index((TextField("f0"),))
+    for x in range(1, 32):
+        client.ft().alter_schema_add((TextField(f"f{x}"),))
+
+    # Should be too many indexes
+    with pytest.raises(redis.ResponseError):
+        client.ft().alter_schema_add((TextField(f"f{x}"),))
+
+    client.ft().dropindex("idx")
+    # Creating the index definition
+    client.ft().create_index((TextField("f0"),), max_text_fields=True)
+    # Fill the index with fields
+    for x in range(1, 50):
+        client.ft().alter_schema_add((TextField(f"f{x}"),))
+
+
 @pytest.mark.redismod
 @skip_ifmodversion_lt("2.0.0", "search")
 def test_create_client_definition(client):
@@ -1068,10 +1245,7 @@ def test_create_client_definition_hash(client):
     Create definition with IndexType.HASH as index type (ON HASH),
     and use hset to test the client definition.
     """
-    definition = IndexDefinition(
-        prefix=["hset:", "henry"],
-        index_type=IndexType.HASH
-    )
+    definition = IndexDefinition(prefix=["hset:", "henry"], index_type=IndexType.HASH)
     createIndex(client.ft(), num_docs=500, definition=definition)
 
     info = client.ft().info()
@@ -1114,15 +1288,10 @@ def test_fields_as_name(client):
     client.ft().create_index(SCHEMA, definition=definition)
 
     # insert json data
-    res = client.json().set(
-        "doc:1",
-        Path.rootPath(),
-        {"name": "Jon", "age": 25}
-    )
+    res = client.json().set("doc:1", Path.rootPath(), {"name": "Jon", "age": 25})
     assert res
 
-    total = client.ft().search(
-        Query("Jon").return_fields("name", "just_a_number")).docs
+    total = client.ft().search(Query("Jon").return_fields("name", "just_a_number")).docs
     assert 1 == len(total)
     assert "doc:1" == total[0].id
     assert "Jon" == total[0].name
@@ -1148,14 +1317,12 @@ def test_search_return_fields(client):
     client.ft().create_index(SCHEMA, definition=definition)
     waitForIndex(client, "idx")
 
-    total = client.ft().search(
-        Query("*").return_field("$.t", as_field="txt")).docs
+    total = client.ft().search(Query("*").return_field("$.t", as_field="txt")).docs
     assert 1 == len(total)
     assert "doc:1" == total[0].id
     assert "riceratops" == total[0].txt
 
-    total = client.ft().search(
-        Query("*").return_field("$.t2", as_field="txt")).docs
+    total = client.ft().search(Query("*").return_field("$.t2", as_field="txt")).docs
     assert 1 == len(total)
     assert "doc:1" == total[0].id
     assert "telmatosaurus" == total[0].txt
@@ -1173,17 +1340,10 @@ def test_synupdate(client):
     )
 
     client.ft().synupdate("id1", True, "boy", "child", "offspring")
-    client.ft().add_document(
-        "doc1",
-        title="he is a baby",
-        body="this is a test")
+    client.ft().add_document("doc1", title="he is a baby", body="this is a test")
 
     client.ft().synupdate("id1", True, "baby")
-    client.ft().add_document(
-        "doc2",
-        title="he is another baby",
-        body="another test"
-    )
+    client.ft().add_document("doc2", title="he is another baby", body="another test")
 
     res = client.ft().search(Query("child").expander("SYNONYM"))
     assert res.docs[0].id == "doc2"
@@ -1225,15 +1385,12 @@ def test_create_json_with_alias(client):
     """
     definition = IndexDefinition(prefix=["king:"], index_type=IndexType.JSON)
     client.ft().create_index(
-        (TextField("$.name", as_name="name"),
-         NumericField("$.num", as_name="num")),
-        definition=definition
+        (TextField("$.name", as_name="name"), NumericField("$.num", as_name="num")),
+        definition=definition,
     )
 
-    client.json().set("king:1", Path.rootPath(), {"name": "henry",
-                                                  "num": 42})
-    client.json().set("king:2", Path.rootPath(), {"name": "james",
-                                                  "num": 3.14})
+    client.json().set("king:1", Path.rootPath(), {"name": "henry", "num": 42})
+    client.json().set("king:2", Path.rootPath(), {"name": "james", "num": 3.14})
 
     res = client.ft().search("@name:henry")
     assert res.docs[0].id == "king:1"
@@ -1260,12 +1417,12 @@ def test_json_with_multipath(client):
     """
     definition = IndexDefinition(prefix=["king:"], index_type=IndexType.JSON)
     client.ft().create_index(
-        (TagField("$..name", as_name="name")),
-        definition=definition
+        (TagField("$..name", as_name="name")), definition=definition
     )
 
-    client.json().set("king:1", Path.rootPath(),
-                      {"name": "henry", "country": {"name": "england"}})
+    client.json().set(
+        "king:1", Path.rootPath(), {"name": "henry", "country": {"name": "england"}}
+    )
 
     res = client.ft().search("@name:{henry}")
     assert res.docs[0].id == "king:1"
@@ -1283,9 +1440,11 @@ def test_json_with_multipath(client):
 def test_json_with_jsonpath(client):
     definition = IndexDefinition(index_type=IndexType.JSON)
     client.ft().create_index(
-        (TextField('$["prod:name"]', as_name="name"),
-         TextField('$.prod:name', as_name="name_unsupported")),
-        definition=definition
+        (
+            TextField('$["prod:name"]', as_name="name"),
+            TextField("$.prod:name", as_name="name_unsupported"),
+        ),
+        definition=definition,
     )
 
     client.json().set("doc:1", Path.rootPath(), {"prod:name": "RediSearch"})
@@ -1304,12 +1463,61 @@ def test_json_with_jsonpath(client):
     res = client.ft().search(Query("@name:RediSearch").return_field("name"))
     assert res.total == 1
     assert res.docs[0].id == "doc:1"
-    assert res.docs[0].name == 'RediSearch'
+    assert res.docs[0].name == "RediSearch"
 
     # return of an unsupported field fails
-    res = client.ft().search(Query("@name:RediSearch")
-                             .return_field("name_unsupported"))
+    res = client.ft().search(Query("@name:RediSearch").return_field("name_unsupported"))
     assert res.total == 1
     assert res.docs[0].id == "doc:1"
     with pytest.raises(Exception):
         res.docs[0].name_unsupported
+
+
+@pytest.mark.redismod
+def test_profile(client):
+    client.ft().create_index((TextField("t"),))
+    client.ft().client.hset("1", "t", "hello")
+    client.ft().client.hset("2", "t", "world")
+
+    # check using Query
+    q = Query("hello|world").no_content()
+    res, det = client.ft().profile(q)
+    assert det["Iterators profile"]["Counter"] == 2.0
+    assert len(det["Iterators profile"]["Child iterators"]) == 2
+    assert det["Iterators profile"]["Type"] == "UNION"
+    assert det["Parsing time"] < 0.5
+    assert len(res.docs) == 2  # check also the search result
+
+    # check using AggregateRequest
+    req = (
+        aggregations.AggregateRequest("*")
+        .load("t")
+        .apply(prefix="startswith(@t, 'hel')")
+    )
+    res, det = client.ft().profile(req)
+    assert det["Iterators profile"]["Counter"] == 2.0
+    assert det["Iterators profile"]["Type"] == "WILDCARD"
+    assert det["Parsing time"] < 0.5
+    assert len(res.rows) == 2  # check also the search result
+
+
+@pytest.mark.redismod
+def test_profile_limited(client):
+    client.ft().create_index((TextField("t"),))
+    client.ft().client.hset("1", "t", "hello")
+    client.ft().client.hset("2", "t", "hell")
+    client.ft().client.hset("3", "t", "help")
+    client.ft().client.hset("4", "t", "helowa")
+
+    q = Query("%hell% hel*")
+    res, det = client.ft().profile(q, limited=True)
+    assert (
+        det["Iterators profile"]["Child iterators"][0]["Child iterators"]
+        == "The number of iterators in the union is 3"
+    )
+    assert (
+        det["Iterators profile"]["Child iterators"][1]["Child iterators"]
+        == "The number of iterators in the union is 4"
+    )
+    assert det["Iterators profile"]["Type"] == "INTERSECT"
+    assert len(res.docs) == 3  # check also the search result
